#!/usr/bin/perl -w
#
# Sensor for gathering PosgreSQL performance data for Zabbix
# Copyright (c) 2008 Jakov Sosic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Changes and Modifications
# * Mon Apr 27 19:00:56 CEST 2009
# 	- fixed for PostgreSQL 8.3,
# 	  http://www.mail-archive.com/pgsql-hackers@postgresql.org/msg99007.html
# 	- rewritten sender to use only one connection
# * Tue Mar 17 17:58:50 CET 2009
# 	- fixed alarms
# * Thu May 15 03:40:39 CET 2008
# 	- script created

use strict;
use warnings;
use DBI;

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $send_file="/var/tmp/zabbixSenderFilePostgreSQL";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd -i $send_file";
my $inputString = '';
my $hostname = `/bin/hostname -f`; 
chomp($hostname);

# signal handling
$SIG{'ALRM'} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
	exit(-1);
};

local $SIG{TERM} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
	exit(-1);
};

alarm 25;

# PostgreSQL access data
my $dbhost = $ENV{'dbhost'} || '127.0.0.1';
my $dbname = $ENV{'dbname'} || 'template1';
my $dbuser = $ENV{'dbuser'} || 'zabbix_agentd';
my $dbpass = $ENV{'dbpass'} || 'p4ssw0rdZabbi><';

# Database connection
    my $Con = "DBI:Pg:dbname=$dbname;host=$dbhost";
    my $Dbh = DBI->connect ($Con,
                            $dbuser,
			    $dbpass,
			    {RaiseError =>1}) || do { print "0\n"; die "Unable to access Database $dbname on host $dbhost as user $dbuser.\nError returned was: ". $DBI::errstr; };
 
# Locks & exclusive locks
    my $sql="SELECT mode,COUNT(mode) FROM pg_locks GROUP BY mode ORDER BY mode;";
    my $sth;
    $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
    $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
    my $locks = 0;
    my $exlocks = 0;
    while (my ($mode, $count) = $sth->fetchrow ()) {
	if ($mode =~ /exclusive/i) {
	    $exlocks = $exlocks + $count;
	}
	$locks = $locks+$count;
    }
    #print "postgresql.locks $locks\n";
    #print "postgresql.exlocks $exlocks\n";

# Number of active processes
    $sql="select sum(numbackends) from pg_stat_database;";
    $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
    $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
    my $activeproc = $sth->fetchrow();
    #print "postgresql.activeproc $activeproc\n";

# Commited and rolled-back transactions
   $sql="select sum(xact_commit) from pg_stat_database;";
   $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
   $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
   my $commit = $sth->fetchrow();
   #print "postgresql.commit $commit\n";
   $sql="select sum(xact_rollback) from pg_stat_database;";
   $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
   $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
   my $rollback = $sth->fetchrow();
   #print "postgresql.rollback $rollback\n";
# Tablespace sizes
   my $pg_default=0;
   $sql="SELECT spcname, PG_TABLESPACE_SIZE(oid) FROM pg_tablespace WHERE spcname='pg_default';";
   $sth = $Dbh->prepare($sql) or do { print "0\n"; die($Dbh->errstr); };
   $sth->execute() or do { print "0\n"; die($Dbh->errstr); };
   my ($mode, $count) = $sth->fetchrow ();
   $pg_default=$count;

# Number of processes
   my $numprocs = 0;
   open(PS_F, "ps -U postgres|grep -v TTY|");
   while (<PS_F>) {
      $numprocs = $numprocs + 1;
   }
   close(PS_F); 
   #my $procnum = system("ps -U postgres | grep -v TTY | wc -l");

# write data to variable
$inputString .= $hostname ." postgresql.locks ".	$locks		."\n" if ($locks);
$inputString .= $hostname ." postgresql.exlocks ".	$exlocks	."\n" if ($exlocks);
$inputString .= $hostname ." postgresql.commit ".	$commit		."\n" if ($commit);
$inputString .= $hostname ." postgresql.rollback ".	$rollback	."\n" if ($rollback);
$inputString .= $hostname ." postgresql.pg_default ".	$pg_default	."\n" if ($pg_default);
$inputString .= $hostname ." postgresql.numprocs ".	$numprocs	."\n" if ($numprocs);

# write everything to file
open FH, ">", $send_file or die("Can not open file $send_file!");
print FH $inputString;
close(FH);

# finally, send the data
if ( qx($zabbix_send_command) =~ /Failed(:)? 0/i ) {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("1\n");
   exit(0);
} else {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("0\n");
   exit(-1);
}
#EOF
